/***************************************************************************
 * Programa que implementa funcionalidades de administração e consultas a  *
 * um banco de dados de linhas de ônibus, o qual, além das linhas,         *
 * armazena empresas, horários, itinerários e logradouros.                 *
 ***************************************************************************
 * Desenvolvido por Adriano Pereira para a disciplina de Fundamentos de    *
 * Banco de Dados - II Sem 2008 - UFSM - Curso de Ciência da Computação    *
 ***************************************************************************
 * Adriano Pereira
 * 2701049
 * peradri@gmail.com
 */


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

/**
   Classe principal de manipulação dos dados com o banco.
   Baseada em exercício feito em aula
   @author Adriano Pereira
 */
public class AcessaBD {

   // Constantes para conexao com HSQLSB (modo standalone)
   static final String JDBC_DRIVER = "org.hsqldb.jdbcDriver";
   static final String DB_URL = "jdbc:hsqldb:file:linhas";
   static final String DB_USER = "sa";
   static final String DB_PASSWD = "";

   Connection conexao;
   /**
      Construtor. "Cria" a conexão com o banco de dados, que no caso será um arquivo
   */
   public AcessaBD() throws Exception {

      // Carrega o driver JDBC do HSQLDB.
      // O driver hsqldb.jar deve estar no classpath.
      Class.forName("org.hsqldb.jdbcDriver");

      // Conecta o programa com o banco de dados. 
      // Os arquivos do banco sao criados se nao existirem.

      conexao = DriverManager.getConnection(DB_URL,
                                            DB_USER,
                                            DB_PASSWD);
   }

   /**
      Fecha o banco de dados e termina a conexao.
   */
   public void shutdown() throws SQLException {
      Statement st = conexao.createStatement();
      st.execute("SHUTDOWN");
      conexao.close();
   }

   /**
      Executa comandos CREATE, DROP, INSERT e UPDATE.
      @param expression Expressão de "manipulação" com o banco de dados
   */
   public synchronized void update(String expression) throws SQLException {
      Statement st = conexao.createStatement();
      st.executeUpdate(expression);
      st.close();
   }

   /**
      Executa comandos SELECT.
      @param expression Expressão de consulta ao banco de dados
      @return rs, ResultSet contendo os dados recuperados pela consulta
   */
   public synchronized ResultSet query(String expression) throws SQLException {
      ResultSet rs = null;
      Statement st = conexao.createStatement();
      rs = st.executeQuery(expression);
      //dump(rs);
      st.close();
      return rs;
   }

   /**
      Imprime os resultados de uma consulta.
      @param rs objeto de ResultSet contendo os dados a serem impressos
   */
   public static void dump(ResultSet rs) throws SQLException {
      ResultSetMetaData meta = rs.getMetaData(); // descritores do ResultSet
      int colmax = meta.getColumnCount(); // numero de colunas do ResultSet
      Object obj;
      //System.out.println("Colmax: "+colmax);
      // rs.next() referencia a proxima linha (true se existir, false em caso contrario)
      for (int i = 1; i <= colmax; i++){
         System.out.print(meta.getColumnName(i) + " ");
      }
      System.out.println("");
      while (rs.next()) {
         for (int i = 0; i < colmax; ++i) {
            obj = rs.getObject(i + 1); // Em SQL a primeira coluna tem indice 1
            System.out.print(obj.toString() + " ");
         }
         System.out.println(" ");
      }
   }
   
   /**
      Cria as tabelas no banco de dados
   */
   private void criaTabelas(){
      try {
         this.update("CREATE TABLE public.Logradouro (idLogradouro IDENTITY NOT NULL, nomeLogradouro VARCHAR(255) NOT NULL, CONSTRAINT Logradouro_pk PRIMARY KEY (idLogradouro));");
      } catch (SQLException e) { /*System.out.println("Nao foi possivel criar a tabela logradouro");*/}
      try {
         this.update("CREATE TABLE public.Empresa (idEmpresa IDENTITY NOT NULL, nomeEmpresa VARCHAR(255) NOT NULL, telefone VARCHAR(10) NOT NULL, site VARCHAR(127) NOT NULL, endereco VARCHAR(255) NOT NULL, CONSTRAINT Empresa_pk PRIMARY KEY (idEmpresa));");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel criar a tabela empresa");*/}
      try {
         this.update("CREATE TABLE public.Linha (idLinha IDENTITY NOT NULL, nome VARCHAR(127) NOT NULL, idEmpresa INTEGER NOT NULL, CONSTRAINT Linha_pk PRIMARY KEY (idLinha));");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel criar a tabela linha");*/}
      try {
         this.update("CREATE TABLE public.Horario (idHorario IDENTITY NOT NULL, hora TIME NOT NULL, idLinha INTEGER NOT NULL, CONSTRAINT Horario_pk PRIMARY KEY (idHorario));");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel criar a tabela horario");*/}
      try {
         this.update("CREATE TABLE public.Itinerario ( idLinha INTEGER NOT NULL, ordem INTEGER NOT NULL, idLogradouro INTEGER NOT NULL, CONSTRAINT Itinerario_pk PRIMARY KEY (idLinha, ordem));");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel criar a tabela itinerario");*/}
      // Altera tabelas
      try {
         this.update("ALTER TABLE public.Itinerario ADD CONSTRAINT Logradouro_Itinerario_fk FOREIGN KEY (idLogradouro) REFERENCES public.Logradour (idLogradouro);");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel alterar a tabela itinerario");*/}
      try {
         this.update("ALTER TABLE public.Linha ADD CONSTRAINT Empresa_Linha_fk FOREIGN KEY (idEmpresa) REFERENCES public.Empresa (idEmpresa);");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel alterar a tabela linha");*/}
      try {
         this.update("ALTER TABLE public.Itinerario ADD CONSTRAINT Linha_Itinerario_fk FOREIGN KEY (idLinha) REFERENCES public.Linha (idLinha);");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel alterar a tabela itinerario");*/}
      try {
         this.update("ALTER TABLE public.Horario ADD CONSTRAINT Linha_Horario_fk FOREIGN KEY (idLinha) REFERENCES public.Linha (idLinha);");
      }catch (SQLException e) { /*System.out.println("Nao foi possivel alterar a tabela horario");*/}
   }

   public static void main(String[] args) {

      try {
         AcessaBD db = new AcessaBD();
         
         db.criaTabelas();

         Empresa emp = new Empresa(db);
         Linha ln = new Linha(db);
         Logradouro lg = new Logradouro(db);
         Horario hr = new Horario(db);
         Itinerario it = new Itinerario(db);
         Interacao interation = new Interacao();
         interation.interage(); // Chama função de interação com a interface administrativa
         db.shutdown();
      } catch (SQLException e1) {
            e1.printStackTrace();
      } catch (Exception e2) {
            e2.printStackTrace();
      }
   }
}
